/*
TempBan-Plugin by Sir D. (21.07.2007)

- stores time bans in adminmod/config/temp_banned.cfg
- will recreate time bans by recalculating remaining 
  bantime after server crash when banned player connect
- command: admin_tempban <"player name"> <bantime>
- command: admin_tempban_list: displays a list of currently temporary banned players!
- command: admin_ban_list: displays a list of currently permanent banned players!

Hints:
- PLUGIN MUST BE THE FIRST PLUGIN IN PLUGINS.INI
- file_access_read and file_access_write must set to 1 in adminmod.cfg
- ACCESS_BAN is needed to execute command by admin
- IT CAN'T be used on LAN. It only stores STEAM_IDs
(maybe change line 120 and 163 the word "bBanByID" with "bBanByIP" get working it for LAN)


Changelog:
1.00:(28.03.2009)
	- added log data of permanent bans: command admin_ban_list
0.98:(28.02.2009)
	- added check for correct steam_id
	- changed code for admin_temp_list
	- changed code to hook for admin_ban and check for timeban.
	
0.95:(27.02.2009)
	- added check for banned players. Only Admin with high accesslevel can overwrite temp bans. Search for "OVERWRITE_ACCESS" in sma!
	- added hook for admin_ban function if time ban is used with admin_ban
	- changed fucntion to temp ban players by steam_id who are NOT online 
	- increased MAX_ENTRIES to 50
	
0.92:(10.01.2009)
	- new command admin_tempban_list to see temp_banned.cfg
	
0.91:(21.07.2007)
	- new messages in chat area for player and admins
	- changed command admin_tempban to <"player name"> <bantime>
	
0.9: 
	- data is loaded in RAM and checked on players connect for temp bans
	
0.8: 
	- data stored in temp file and checked file for temp bans in this file
	- BUG: remaining 59 seconds time ban gives players a permanent ban! :(	
*/

#include <string>
#include <admin>
#include <adminlib>


//###########################################################
#define MAX_ENTRIES		50
#define OVERWRITE_ACCESS	131071
//###########################################################


new STRING_VERSION[MAX_TEXT_LENGTH]="AM 2.50.60_1.0";
new g_Userbancfg[MAX_TEXT_LENGTH]="addons/adminmod/config/temp_banned.cfg";
new g_Userpermbancfg[MAX_TEXT_LENGTH]="addons/adminmod/config/perm_banned.cfg";
new g_Specmode[MAX_PLAYERS];

new g_AuthID[MAX_ENTRIES][MAX_AUTHID_LENGTH];
new g_Info[MAX_ENTRIES][MAX_TEXT_LENGTH];
new g_FileTimeStamp[MAX_ENTRIES];
new g_BanTime[MAX_ENTRIES];
new g_loaded;

public plugin_init() {
	plugin_registerinfo("TempBan-Plugin","Bans player temporarly and stores data in temp_banned.cfg",STRING_VERSION);
	plugin_registercmd("admin_ban","check_ban",ACCESS_BAN);
	plugin_registercmd("admin_tempban","check_ban",ACCESS_BAN,"admin_tempban <playername> <bantime>: bans player for x minutes");
	plugin_registercmd("admin_tempban_list","admin_tempban_list",ACCESS_BAN,"admin_tempban_list: displays current temp ban list");
	plugin_registercmd("admin_ban_list","admin_ban_list",ACCESS_BAN,"admin_ban_list: displays current permanent ban list");
	plugin_registercmd("specmode","specmode",ACCESS_ALL);
	load_file();
	return PLUGIN_CONTINUE;
}

load_file(){
	if(fileexists(g_Userbancfg)){
		new Data[MAX_DATA_LENGTH];
		new Info[MAX_DATA_LENGTH];
		new FileTimeStamp[MAX_NUMBER_LENGTH];
		new AuthID[MAX_AUTHID_LENGTH];
		new BanTime[MAX_NUMBER_LENGTH];
		new iFileSize;
		new i;
		new filetimestamp;
		new bantime;
		
		iFileSize=filesize(g_Userbancfg);
		for(i=1;i<=iFileSize;i++){
			readfile(g_Userbancfg,Data,i,MAX_DATA_LENGTH);
			if(g_loaded<MAX_ENTRIES){
				if(strncmp(Data,"//",2)!=0){
					strsep(Data,"",FileTimeStamp,MAX_NUMBER_LENGTH,AuthID,MAX_AUTHID_LENGTH,BanTime,MAX_NUMBER_LENGTH,Info,MAX_TEXT_LENGTH);

					//get old timestamp of file as int
					filetimestamp=strtonum(FileTimeStamp);
					//Calculate new bantime in seconds as int
					bantime=strtonum(BanTime)*60-(systemtime()-filetimestamp);

					if(bantime<=0){
						//comment out line, he can play again
						snprintf(Data,MAX_DATA_LENGTH,"//%s",Data);
						writefile(g_Userbancfg,Data,i);
					}else{
						//store in RAM to make a faster check for STEAM_ID, if temp banned player connects after server crash
						strcpy(g_AuthID[g_loaded],AuthID,MAX_AUTHID_LENGTH);
						strcpy(g_Info[g_loaded],Info,MAX_DATA_LENGTH);
						//as int
						g_FileTimeStamp[g_loaded]=filetimestamp;
						//as int, back to minutes
						g_BanTime[g_loaded]=bantime/60;
						g_loaded++;
					}
				}
			}else{
				selfmessage("[PLUGIN TEMP-BAN] Max. entries reached. Please check plugin_temp_ban.sma file and increase MAX_ENTRIES. Recompile plugin afterwards.");
			}
		}
		g_loaded--;
		snprintf(Data,MAX_DATA_LENGTH,"[PLUGIN TEMP-BAN] Currently %i time bans are loaded!",g_loaded+1);
		selfmessage(Data);
	}else{
		selfmessage("[PLUGIN TEMP-BAN] File addons/adminmod/config/temp_banned.cfg not found!");
	}
}

check_tempban(AuthID[],&bantime){
	new i;
	for(i=0;i<=g_loaded;i++){
		if(strcmp(g_AuthID[i],AuthID)==0){
			bantime=g_BanTime[i];
			break;
		}
	}
}

issteamid(SteamID[]){
	if (strncasecmp(SteamID,"Steam_",6) == 0){
		return 1;
	}
	return 0;
}

timeban(User[],Player[],bantime){
	new Text[MAX_TEXT_LENGTH];
	new TargetName[MAX_NAME_LENGTH];
	new AuthID[MAX_AUTHID_LENGTH];
	new timestamp;
	new oldbantime;

	/* no bantime, do nothing */
	if(!bantime){
		selfmessage("[PLUGIN TEMP-BAN] No bantime found. Maybe use quotes around playername that bantime can be identified properly!");
	}else{

		if (check_user(Player)) {
			get_username(Player,TargetName,MAX_NAME_LENGTH);
			if(check_immunity(TargetName)) {
				snprintf(Text, MAX_TEXT_LENGTH, "[PLUGIN TEMP-BAN] Laf. You can't ban '%s', you silly bear.", TargetName);
				selfmessage(Text);
			}else{
				get_userAuthID(TargetName,AuthID);
			}
		}else{
			//is steam_id available
			if(strlen(AuthID)==0){
				strcpy(AuthID,Player,MAX_AUTHID_LENGTH);
			}
			
			//is steam_id valid?
			if(!issteamid(AuthID)){
				snprintf(Text,MAX_TEXT_LENGTH,"[PLUGIN TEMP-BAN] Steam_ID %s seems not to be valid! Please try again!",AuthID);
				selfmessage(Text);
			}else{
				//check, if old bantimes exists for this AuthID
				check_tempban(AuthID,oldbantime);
				if(oldbantime>0){
					snprintf(Text,MAX_TEXT_LENGTH,"[PLUGIN TEMP-BAN] It still exists a timeban of %s for %i minutes!",AuthID,oldbantime);
					selfmessage(Text);
					if(access(OVERWRITE_ACCESS,User)){
						snprintf(Text,MAX_TEXT_LENGTH,"[PLUGIN TEMP-BAN] You are allowed to change it to %i minutes!",AuthID,bantime);
						selfmessage(Text);	
					}else{
						snprintf(Text,MAX_TEXT_LENGTH,"[PLUGIN TEMP-BAN] You are NOT allowed to change it!");
						selfmessage(Text);
						return;
					}
				}

				/*write it to file*/
				timestamp=systemtime();
				snprintf(Text,MAX_TEXT_LENGTH,"%i%s%i%sAdmin:%s",timestamp,AuthID,bantime,TargetName,User);
				writefile(g_Userbancfg,Text,-1);

				/*store it in RAM*/
				if(g_loaded<MAX_ENTRIES){
					g_loaded++;
					strcpy(g_AuthID[g_loaded],AuthID,MAX_AUTHID_LENGTH);
					snprintf(g_Info[g_loaded],MAX_TEXT_LENGTH,"-%s-%s",TargetName,User);
					g_FileTimeStamp[g_loaded]=timestamp;
					g_BanTime[g_loaded]=bantime;
				}
				do_timeban(AuthID,bantime);
			}
		}
	}
}

do_timeban(AuthID[],bantime){
	new Data[MAX_DATA_LENGTH];
	snprintf(Data,MAX_DATA_LENGTH,"[PLUGIN TEMP-BAN] Time ban created for %s: %i min till unban]",AuthID,bantime);
	say(Data);
	messageex(AuthID,Data,print_console);
	ban(AuthID,bantime,bBanByID);
}



public plugin_disconnect(HLUserName, UserIndex){
	g_Specmode[UserIndex]=0;
	return PLUGIN_CONTINUE;
}

//Player enters game
public specmode(HLCommand,HLData,HLUserName,UserIndex) {
	//only check, if g_loaded >= 0 -> temp ban exists.
	if(!g_Specmode[UserIndex] && g_loaded!=-1){
		new Data[MAX_DATA_LENGTH];
		new User[MAX_NAME_LENGTH];
		new AuthID[MAX_AUTHID_LENGTH];
		new bantime;
		convert_string(HLData,Data,MAX_DATA_LENGTH);
		convert_string(HLUserName,User,MAX_DATA_LENGTH);
		get_userAuthID(User,AuthID);
		
		//if server has crashed, check if time ban still exists for player's steam_id 
		check_tempban(AuthID,bantime);
		if(bantime>0){
			do_timeban(AuthID,bantime);
		}
		g_Specmode[UserIndex]=1;
	}
	return PLUGIN_CONTINUE;
}

//hook admin_ban to see, if it is a timeban!
public check_ban(HLCommand,HLData,HLUserName,UserIndex) {
	new Data[MAX_DATA_LENGTH];
	new Command[MAX_COMMAND_LENGTH];
	new User[MAX_NAME_LENGTH];
	new strTime[MAX_NUMBER_LENGTH];
	new strReason[MAX_TEXT_LENGTH];
	new bantime;
	new ban_user[MAX_DATA_LENGTH];
	
	//Log Data
	new ST[MAX_DATA_LENGTH];
	new iTargetUserIndex;
	new iTargetSessionID;
	new iTargetTeam;
	new TargetName[MAX_NAME_LENGTH];
	new TargetAuthID[MAX_AUTHID_LENGTH];
	new TargetIP[MAX_NAME_LENGTH];
	//
	
	convert_string(HLCommand,Command,MAX_COMMAND_LENGTH);
	convert_string(HLData,Data,MAX_DATA_LENGTH);
	convert_string(HLUserName,User,MAX_DATA_LENGTH);
	
	strsep(Data," ",ban_user,MAX_DATA_LENGTH,strTime,MAX_TEXT_LENGTH,strReason,MAX_TEXT_LENGTH);
	
	bantime=strtonum(strTime);
	
	//if it is a banbyIP or has no strTime, let it pass to admin_ban of plugin_base BUT make Log entry
	if(getvar("sv_lan")!=0 || strcasecmp(Command,"admin_banip")==0 || bantime==0){
		if(check_user(ban_user)) {
			get_username(ban_user,TargetName,MAX_NAME_LENGTH);
			if(check_immunity(TargetName)!=1) {
				get_userindex(TargetName,iTargetUserIndex);
				if(playerinfo(iTargetUserIndex,TargetName,MAX_NAME_LENGTH,iTargetSessionID,_,iTargetTeam,_,TargetAuthID)){
					get_userIP(TargetName,TargetIP,MAX_NAME_LENGTH);
					servertime(ST, MAX_DATA_LENGTH, "L %m/%d/%y - %H:%M:%S:");
					snprintf(Data,MAX_DATA_LENGTH,"%s admin %s banned %s<%i><%s><%i><%s> - reason: %s",ST,User,TargetName,iTargetSessionID,TargetAuthID,iTargetTeam,TargetIP,strReason);
					writefile(g_Userpermbancfg,Data,-1);
				}
			}
		}
		return PLUGIN_CONTINUE;
	}
	timeban(User,ban_user,bantime);
	return PLUGIN_HANDLED;
}

public admin_tempban_list(HLCommand,HLData,HLUserName,UserIndex) {
	new Data[MAX_DATA_LENGTH];
	new i;
	selfmessage("[PLUGIN TEMP-BAN LIST]");
	for(i=0;i<=g_loaded;i++){
		snprintf(Data,MAX_DATA_LENGTH,"* %s - ban remains for %i minutes - Info: %s",g_AuthID[i],g_BanTime[i],g_Info[i]);
		selfmessage(Data);
	}
	return PLUGIN_HANDLED;
}

public admin_ban_list(HLCommand,HLData,HLUserName,UserIndex) {
	if(fileexists(g_Userpermbancfg)){
		new Data[MAX_DATA_LENGTH];
		new iFileSize;
		new i;
		
		selfmessage("[PERMANENT BAN LIST]");
		iFileSize=filesize(g_Userpermbancfg);
		for(i=1;i<=iFileSize;i++){
			readfile(g_Userpermbancfg,Data,i,MAX_DATA_LENGTH);
			selfmessage(Data);
		}
	}else{
		selfmessage("[PLUGIN TEMP BAN]");
		selfmessage("File perm_banned.cfg could not be found!");
	}			
	return PLUGIN_HANDLED;
}



